// SPort driver code by Mike Blandford

#include <Arduino.h>
#include "SoftwareSerial.h"

#include "OXS_OUT_FRSKY.h"

extern uint16_t Xanalog ;
extern uint16_t Analog ;

OXS_OUT_FRSKY::OXS_OUT_FRSKY(uint8_t pinTx/*,HardwareSerial &print*/) : 
_mySerial(/*pinTx,pinTx,invert=*/true)
{
}

// **************** Setup the FRSky OutputLib *********************
void OXS_OUT_FRSKY::setup()
{
  _mySerial.begin();
  _mySerial.listen();
}

bool OXS_OUT_FRSKY::timeToSend()
{
  static uint8_t lastRx = 0;

  while (_mySerial.available())
	{
    int rx = _mySerial.read();
    if (lastRx == 0x7e && rx == SENSOR_ID)
		{
      lastRx = 0;
      return true;
    }
    lastRx = rx;
  }

  return false;
}

void OXS_OUT_FRSKY::sendByte(uint8_t byte)
{
	// CRC update
  crc += byte; //0-1FF
  crc += crc >> 8; //0-100
  crc &= 0x00ff;
  crc += crc >> 8; //0-0FF
  crc &= 0x00ff;

	if ( ( byte == 0x7E ) || ( byte == 0x7D ) )
	{
  	_mySerial.write( 0x7D );
		
		byte &= ~0x20 ;
	}
  _mySerial.write(byte);
}

void OXS_OUT_FRSKY::sendCrc()
{
  sendByte(0xFF-crc) ;
}

void OXS_OUT_FRSKY::sendValue(uint16_t id, uint32_t value)
{
	_mySerial.setTX() ;
	
  crc = 0;
  sendByte(0x10); // DATA_FRAME
  uint8_t *bytes = (uint8_t*)&id;
  sendByte(bytes[0]);
  sendByte(bytes[1]);
  bytes = (uint8_t*)&value;
  sendByte(bytes[0]);
  sendByte(bytes[1]);
  sendByte(bytes[2]);
  sendByte(bytes[3]);
  sendCrc();
	_mySerial.setRX() ;
}

#define A2_ID       0xF103

void OXS_OUT_FRSKY::sendData()
{
  sendValue( A2_ID, ((uint32_t)Xanalog << 16) | Analog ) ;
}


